@@ -71,6 +71,8 @@ class JunebugApi(object):
         yield Channel.start_all_channels(
             self.redis, self.config, self.service, self.plugins)
 
+        yield Router.start_all_routers(self)
+
         if self.config.rabbitmq_management_interface:
             self.rabbitmq_management_client = RabbitmqManagementClient(
                 self.config.rabbitmq_management_interface,
@@ -156,10 +158,6 @@ class JunebugApi(object):
     @inlineCallbacks
     def create_channel(self, request, body):
         '''Create a channel'''
-        if not (body.get('mo_url') or body.get('amqp_queue')):
-            raise ApiUsageError(
-                'One or both of "mo_url" and "amqp_queue" must be specified')
-
         channel = Channel(
             self.redis, self.config, body, self.plugins)
         yield channel.start(self.service)
@@ -269,26 +267,17 @@ class JunebugApi(object):
     @inlineCallbacks
     def send_message(self, request, body, channel_id):
         '''Send an outbound (mobile terminated) message'''
-        if 'to' not in body and 'reply_to' not in body:
-            raise ApiUsageError(
-                'Either "to" or "reply_to" must be specified')
-
         channel = yield Channel.from_id(
             self.redis, self.config, channel_id, self.service, self.plugins)
 
-        if 'reply_to' in body:
-            msg = yield channel.send_reply_message(
-                self.message_sender, self.outbounds, self.inbounds, body,
-                allow_expired_replies=self.config.allow_expired_replies)
+        if (channel.has_destination):
+            msg = yield self.send_message_on_channel(channel_id, body)
+
+            returnValue(response(
+                request, 'message submitted', msg, code=http.CREATED))
         else:
-            msg = yield channel.send_message(
-                self.message_sender, self.outbounds, body)
-
-        yield self.message_rate.increment(
-            channel_id, 'outbound', self.config.metric_window)
-
-        returnValue(response(
-            request, 'message submitted', msg, code=http.CREATED))
+            raise ApiUsageError(
+                'This channel has no "mo_url" or "amqp_queue"')
 
     @app.route(
         '/channels/<string:channel_id>/messages/<string:message_id>',
@@ -296,21 +285,16 @@ class JunebugApi(object):
     @inlineCallbacks
     def get_message_status(self, request, channel_id, message_id):
         '''Retrieve the status of a message'''
-        events = yield self.outbounds.load_all_events(channel_id, message_id)
-        events = sorted(
-            (api_from_event(channel_id, e) for e in events),
-            key=lambda e: e['timestamp'])
+        channel = yield Channel.from_id(
+            self.redis, self.config, channel_id, self.service, self.plugins)
 
-        last_event = events[-1] if events else None
-        last_event_type = last_event['event_type'] if last_event else None
-        last_event_timestamp = last_event['timestamp'] if last_event else None
-
-        returnValue(response(request, 'message status', {
-            'id': message_id,
-            'last_event_type': last_event_type,
-            'last_event_timestamp': last_event_timestamp,
-            'events': events,
-        }))
+        if (channel.has_destination):
+            data = yield self.get_message_events(
+                request, channel_id, message_id)
+            returnValue(response(request, 'message status', data))
+        else:
+            raise ApiUsageError(
+                'This channel has no "mo_url" or "amqp_queue"')
 
     @app.route('/routers/', methods=['GET'])
     def get_router_list(self, request):
@@ -336,7 +320,7 @@ class JunebugApi(object):
     @inlineCallbacks
     def create_router(self, request, body):
         """Create a new router"""
-        router = Router(self.router_store, self.config, body)
+        router = Router(self, body)
         yield router.validate_config()
         router.start(self.service)
         yield router.save()
@@ -350,8 +334,7 @@ class JunebugApi(object):
     @app.route('/routers/<string:router_id>', methods=['GET'])
     def get_router(self, request, router_id):
         """Get the configuration details and status of a specific router"""
-        d = Router.from_id(
-            self.router_store, self.config, self.service, router_id)
+        d = Router.from_id(self, router_id)
         d.addCallback(lambda router: router.status())
         d.addCallback(partial(response, request, 'router found'))
         return d
@@ -373,8 +356,7 @@ class JunebugApi(object):
     @inlineCallbacks
     def replace_router_config(self, request, body, router_id):
         """Replace the router config with the one specified"""
-        router = yield Router.from_id(
-            self.router_store, self.config, self.service, router_id)
+        router = yield Router.from_id(self, router_id)
 
         for field in ['type', 'label', 'config', 'metadata']:
             router.router_config.pop(field, None)
@@ -405,8 +387,7 @@ class JunebugApi(object):
     @inlineCallbacks
     def update_router_config(self, request, body, router_id):
         """Update the router config with the one specified"""
-        router = yield Router.from_id(
-            self.router_store, self.config, self.service, router_id)
+        router = yield Router.from_id(self, router_id)
 
         router.router_config.update(body)
         yield router.validate_config()
@@ -421,12 +402,23 @@ class JunebugApi(object):
     @app.route('/routers/<string:router_id>', methods=['DELETE'])
     @inlineCallbacks
     def delete_router(self, request, router_id):
-        router = yield Router.from_id(
-            self.router_store, self.config, self.service, router_id)
+        router = yield Router.from_id(self, router_id)
         yield router.stop()
         yield router.delete()
         returnValue(response(request, 'router deleted', {}))
 
+    @app.route('/routers/<string:router_id>/logs', methods=['GET'])
+    @inlineCallbacks
+    def get_router_logs(self, request, router_id):
+        '''Get the last N logs for a router, sorted reverse
+        chronologically.'''
+        n = request.args.get('n', None)
+        if n is not None:
+            n = int(n[0])
+        router = yield Router.from_id(self, router_id)
+        logs = yield router.get_logs(n)
+        returnValue(response(request, 'logs retrieved', logs))
+
     @app.route('/routers/<string:router_id>/destinations/', methods=['POST'])
     @json_body
     @validate(
@@ -439,7 +431,10 @@ class JunebugApi(object):
                 'mo_url': {'type': 'string'},
                 'mo_url_token': {'type': 'string'},
                 'amqp_queue': {'type': 'string'},
-                'character_limit': {'type': 'int'},
+                'character_limit': {
+                    'type': 'integer',
+                    'minimum': 0,
+                },
             },
             'additionalProperties': False,
             'required': ['config'],
@@ -447,8 +442,7 @@ class JunebugApi(object):
     @inlineCallbacks
     def create_router_destination(self, request, body, router_id):
         """Create a new destination for the router"""
-        router = yield Router.from_id(
-            self.router_store, self.config, self.service, router_id)
+        router = yield Router.from_id(self, router_id)
         yield router.validate_destination_config(body['config'])
 
         destination = router.add_destination(body)
@@ -464,8 +458,7 @@ class JunebugApi(object):
     @app.route('/routers/<string:router_id>/destinations/', methods=['GET'])
     def get_router_destination_list(self, request, router_id):
         """Get the list of destinations for a router"""
-        d = Router.from_id(
-            self.router_store, self.config, self.service, router_id)
+        d = Router.from_id(self, router_id)
         d.addCallback(lambda router: router.get_destination_list())
         d.addCallback(partial(response, request, 'destinations retrieved'))
         return d
@@ -476,8 +469,7 @@ class JunebugApi(object):
     @inlineCallbacks
     def get_destination(self, request, router_id, destination_id):
         """Get the config and status of a destination"""
-        router = yield Router.from_id(
-            self.router_store, self.config, self.service, router_id)
+        router = yield Router.from_id(self, router_id)
         destination = router.get_destination(destination_id)
         returnValue(response(
             request, 'destination found', (yield destination.status())
@@ -497,7 +489,10 @@ class JunebugApi(object):
                 'mo_url': {'type': 'string'},
                 'mo_url_token': {'type': 'string'},
                 'amqp_queue': {'type': 'string'},
-                'character_limit': {'type': 'int'},
+                'character_limit': {
+                    'type': 'integer',
+                    'minimum': 0,
+                },
             },
             'additionalProperties': False,
             'required': ['config'],
@@ -506,8 +501,7 @@ class JunebugApi(object):
     def replace_router_destination(
             self, request, body, router_id, destination_id):
         """Replace the config of a router destination"""
-        router = yield Router.from_id(
-            self.router_store, self.config, self.service, router_id)
+        router = yield Router.from_id(self, router_id)
         yield router.validate_destination_config(body['config'])
 
         destination = router.get_destination(destination_id)
@@ -535,7 +529,10 @@ class JunebugApi(object):
                 'mo_url': {'type': 'string'},
                 'mo_url_token': {'type': 'string'},
                 'amqp_queue': {'type': 'string'},
-                'character_limit': {'type': 'int'},
+                'character_limit': {
+                    'type': 'integer',
+                    'minimum': 0,
+                },
             },
             'additionalProperties': False,
             'required': [],
@@ -544,8 +541,7 @@ class JunebugApi(object):
     def update_router_destination(
             self, request, body, router_id, destination_id):
         """Update the config of a router destination"""
-        router = yield Router.from_id(
-            self.router_store, self.config, self.service, router_id)
+        router = yield Router.from_id(self, router_id)
         if 'config' in body:
             yield router.validate_destination_config(body['config'])
 
@@ -565,8 +561,7 @@ class JunebugApi(object):
     @inlineCallbacks
     def delete_router_destination(self, request, router_id, destination_id):
         """Delete and stop the router destination"""
-        router = yield Router.from_id(
-            self.router_store, self.config, self.service, router_id)
+        router = yield Router.from_id(self, router_id)
         destination = router.get_destination(destination_id)
 
         yield router.stop()
@@ -575,21 +570,121 @@ class JunebugApi(object):
 
         returnValue(response(request, 'destination deleted', {}))
 
+    @app.route(
+        '/routers/<string:router_id>/destinations/<string:destination_id>/messages/',  # noqa
+        methods=['POST'])
+    @json_body
+    @validate(
+        body_schema({
+            'type': 'object',
+            'properties': {
+                'to': {'type': 'string'},
+                'from': {'type': ['string', 'null']},
+                'group': {'type': ['string', 'null']},
+                'reply_to': {'type': 'string'},
+                'content': {'type': ['string', 'null']},
+                'event_url': {'type': 'string'},
+                'event_auth_token': {'type': 'string'},
+                'priority': {'type': 'string'},
+                'channel_data': {'type': 'object'},
+            },
+            'required': ['content'],
+            'additionalProperties': False,
+        }))
+    @inlineCallbacks
+    def send_destination_message(
+            self, request, body, router_id, destination_id):
+        '''Send an outbound (mobile terminated) message'''
+        router = yield Router.from_id(self, router_id)
+        router.get_destination(destination_id)
+        channel_id = yield router.router_worker.get_destination_channel(
+            destination_id, body)
+
+        in_msg = None
+        if 'reply_to' in body:
+            in_msg = yield self.inbounds.load_vumi_message(
+                destination_id, body['reply_to'])
+
+        msg = yield self.send_message_on_channel(
+            channel_id, body, in_msg)
+
+        self.outbounds.store_message(destination_id, msg)
+
+        returnValue(response(
+            request, 'message submitted', msg, code=http.CREATED))
+
+    @app.route(
+        '/routers/<string:router_id>/destinations/<string:destination_id>/messages/<string:message_id>',  # noqa
+        methods=['GET'])
+    @inlineCallbacks
+    def get_destination_message_status(
+            self, request, router_id, destination_id, message_id):
+
+        router = yield Router.from_id(self, router_id)
+        router.get_destination(destination_id)
+
+        data = yield self.get_message_events(
+            request, destination_id, message_id)
+
+        returnValue(response(request, 'message status', data))
+
+    @inlineCallbacks
+    def get_message_events(self, request, location_id, message_id):
+        events = yield self.outbounds.load_all_events(location_id, message_id)
+        events = sorted(
+            (api_from_event(location_id, e) for e in events),
+            key=lambda e: e['timestamp'])
+
+        last_event = events[-1] if events else None
+        last_event_type = last_event['event_type'] if last_event else None
+        last_event_timestamp = last_event['timestamp'] if last_event else None
+
+        returnValue({
+            'id': message_id,
+            'last_event_type': last_event_type,
+            'last_event_timestamp': last_event_timestamp,
+            'events': events,
+        })
+
+    @inlineCallbacks
+    def send_message_on_channel(self, channel_id, body, in_msg=None):
+        if 'to' not in body and 'reply_to' not in body:
+            raise ApiUsageError(
+                'Either "to" or "reply_to" must be specified')
+
+        channel = yield Channel.from_id(
+            self.redis, self.config, channel_id, self.service, self.plugins)
+
+        if 'reply_to' in body:
+            msg = yield channel.send_reply_message(
+                self.message_sender, self.outbounds, self.inbounds, body,
+                allow_expired_replies=self.config.allow_expired_replies,
+                in_msg=in_msg)
+        else:
+            msg = yield channel.send_message(
+                self.message_sender, self.outbounds, body)
+
+        yield self.message_rate.increment(
+            channel_id, 'outbound', self.config.metric_window)
+
+        returnValue(msg)
+
     @app.route('/health', methods=['GET'])
     def health_status(self, request):
         if self.config.rabbitmq_management_interface:
 
-            def get_queues(channel_ids):
+            def get_queues(queue_data):
 
                 gets = []
 
-                for channel_id in channel_ids:
-                    for sub in ['inbound', 'outbound', 'event']:
-                        queue_name = "%s.%s" % (channel_id, sub)
+                for _, queue_ids in queue_data:
+                    for queue_id in queue_ids:
+                        for sub in ['inbound', 'outbound', 'event']:
+                            queue_name = "%s.%s" % (queue_id, sub)
 
-                        get = self.rabbitmq_management_client.get_queue(
-                            self.amqp_config['vhost'], queue_name)
-                        gets.append(get)
+                            get = self.rabbitmq_management_client.get_queue(
+                                self.amqp_config['vhost'], queue_name)
+                            gets.append(get)
 
                 return gets
 
@@ -597,15 +692,15 @@ class JunebugApi(object):
                 queues = []
                 stuck = False
 
-                for result in results:
-                    queue = result[1]
+                for _, queue in results:
 
                     if ('messages' in queue):
                         details = {
                             'name': queue['name'],
                             'stuck': False,
                             'messages': queue.get('messages'),
-                            'rate': queue['messages_details']['rate']
+                            'rate':
+                                queue['messages_stats']['ack_details']['rate']
                         }
                         if (details['messages'] > 0 and details['rate'] == 0):
                             stuck = True
@@ -621,7 +716,27 @@ class JunebugApi(object):
 
                 return response(request, status, queues, code=code)
 
-            d = Channel.get_all(self.redis)
+            def get_routers_objects(router_ids):
+                gets = []
+                for router_id in router_ids:
+                    get = Router.from_id(self, router_id)
+                    gets.append(get)
+
+                return gets
+
+            def get_destinations(routers):
+                destinations = []
+                for _, router in routers:
+                    destinations.extend(router.get_destination_list())
+                return destinations
+
+            d1 = Channel.get_all(self.redis)
+            d2 = Router.get_all(self.router_store)
+            d2.addCallback(get_routers_objects)
+            d2.addCallback(defer.DeferredList)
+            d2.addCallback(get_destinations)
+
+            d = defer.DeferredList([d1, d2])
             d.addCallback(get_queues)
             d.addCallback(defer.DeferredList)
             d.addCallback(return_queue_results)
