@@ -20,14 +20,15 @@ from vumi.service import get_spec
 from vumi.tests.fake_amqp import FakeAMQPBroker, FakeAMQPChannel
 from vumi.tests.helpers import PersistenceHelper
 from vumi.transports import Transport
-from vumi.worker import BaseWorker
 
 import junebug
 from junebug import JunebugApi
 from junebug.amqp import JunebugAMQClient, MessageSender
 from junebug.channel import Channel
 from junebug.plugin import JunebugPlugin
-from junebug.router import InvalidRouterConfig, InvalidRouterDestinationConfig
+from junebug.router import (
+    InvalidRouterConfig, InvalidRouterDestinationConfig, BaseRouterWorker,
+    Router)
 from junebug.service import JunebugService
 from junebug.config import JunebugConfig
 from junebug.stores import MessageRateStore
@@ -142,30 +143,41 @@ class LoggingTestTransport(Transport):
         self.log.msg(message, source=self)
 
 
-class TestRouter(BaseWorker):
+class TestRouter(BaseRouterWorker):
     """Router used for testing the API."""
-    # TODO: Create a proper base class for Junebug routers
+    setup_called = False
+    teardown_called = False
+
     @classmethod
-    def validate_router_config(cls, config):
+    def validate_router_config(cls, api, config, router_id=None):
         """Testing config requires the ``test`` parameter to be ``pass``"""
         if config.get('test') != 'pass':
             raise InvalidRouterConfig('test must be pass')
 
     @classmethod
-    def validate_destination_config(cls, config):
+    def validate_destination_config(cls, api, config):
         """Testing destination config requires the ``target`` parameter to be
         ``valid``"""
         if config.get('target') != 'valid':
             raise InvalidRouterDestinationConfig('target must be valid')
 
-    def setup_connectors(self):
-        pass
+    def setup_router(self):
+        self.setup_called = True
 
-    def setup_worker(self):
-        pass
+    def teardown_router(self):
+        self.teardown_called = True
 
-    def teardown_worker(self):
-        pass
+    def test_log(self, message='Test log'):
+        self.log.msg(message, source=self)
+
+    def get_destination_channel(self, destination_id, message_body):
+        config = self.get_static_config()
+
+        for dest in config.destinations:
+            if dest['id'] == destination_id:
+                return dest['config']['channel']
+
+        return None
 
 
 class JunebugTestBase(TestCase):
@@ -245,6 +257,8 @@ class JunebugTestBase(TestCase):
 
     def create_destination_config(self, **kw):
         properties = deepcopy(self.default_destination_properties)
+        config = kw.pop('config', {})
+        properties['config'].update(config)
         properties.update(kw)
         return properties
 
@@ -285,6 +299,18 @@ class JunebugTestBase(TestCase):
         channel = yield Channel.from_id(redis, config, id, service)
         returnValue(channel)
 
+    @inlineCallbacks
+    def create_test_router(self, service, config={}):
+        self.patch(junebug.logging_service, 'LogFile', DummyLogFile)
+
+        config = self.create_router_config(config=config)
+        router = Router(self.api, config)
+
+        yield router.start(self.service)
+
+        yield router.save()
+        returnValue(router)
+
     @inlineCallbacks
     def get_redis(self):
         '''Creates and returns a redis manager'''
