@@ -10,8 +10,10 @@ from treq.testing import StubTreq
 from treq.testing import RequestSequence, StringStubbingResource
 
 from vumi.message import TransportEvent, TransportUserMessage
+from vumi.tests.helpers import MessageHelper
 
 from junebug.channel import Channel
+from junebug.router.base import Router
 from junebug.utils import api_from_message
 from junebug.tests.helpers import JunebugTestBase, FakeJunebugPlugin
 from junebug.utils import api_from_event, conjoin, omit
@@ -26,6 +28,9 @@ class TestJunebugApi(JunebugTestBase):
         self.patch_logger()
         yield self.start_server()
 
+        self.messagehelper = MessageHelper()
+        self.addCleanup(self.messagehelper.cleanup)
+
     def get(self, url, params={}):
         return treq.get(
             "%s%s" % (self.url, url), params=params, persistent=False)
@@ -73,7 +78,7 @@ class TestJunebugApi(JunebugTestBase):
 
         self.assertEqual(data, {
             'status': code,
-            'code': http.RESPONSES[code],
+            'code': http.RESPONSES.get(code, code),
             'description': description,
             'result': result,
         })
@@ -83,12 +88,12 @@ class TestJunebugApi(JunebugTestBase):
         resp = yield self.get('/foobar')
         yield self.assert_response(
             resp, http.NOT_FOUND,
-            'The requested URL was not found on the server.  If you entered '
+            'The requested URL was not found on the server. If you entered '
             'the URL manually please check your spelling and try again.', {
                 'errors': [{
                     'code': 404,
                     'message': ('404 Not Found: The requested URL was not '
-                                'found on the server.  If you entered the URL'
+                                'found on the server. If you entered the URL'
                                 ' manually please check your spelling and try'
                                 ' again.'),
                     'type': 'Not Found',
@@ -98,20 +103,16 @@ class TestJunebugApi(JunebugTestBase):
     @inlineCallbacks
     def test_redirect_http_error(self):
         resp = yield self.get('/channels')
-        [redirect] = resp.history()
         yield self.assert_response(
-            redirect, http.MOVED_PERMANENTLY,
+            resp, 308,
             None, {
                 'errors': [{
-                    'code': 301,
-                    'message': '301 Moved Permanently: None',
+                    'code': 308,
+                    'message': '308 Permanent Redirect: None',
                     'new_url': '%s/channels/' % self.url,
-                    'type': 'Moved Permanently',
+                    'type': 'Permanent Redirect',
                 }],
             })
-        yield self.assert_response(
-            resp, http.OK,
-            'channels listed', [])
 
     @inlineCallbacks
     def test_invalid_json_handling(self):
@@ -289,24 +290,6 @@ class TestJunebugApi(JunebugTestBase):
                 ])
             })
 
-    @inlineCallbacks
-    def test_create_channel_mo_destination(self):
-        '''When creating a channel, one of or both of mo_url and mo_queue
-        must be present.'''
-        resp = yield self.post('/channels/', {
-            'type': 'smpp',
-            'config': {}
-        })
-        self.maxDiff = None
-        yield self.assert_response(
-            resp, http.BAD_REQUEST, 'api usage error', {
-                'errors': [{
-                    'message': 'One or both of "mo_url" and "amqp_queue" must'
-                               ' be specified',
-                    'type': 'ApiUsageError',
-                }],
-            })
-
     @inlineCallbacks
     def test_get_missing_channel(self):
         resp = yield self.get('/channels/foo-bar')
@@ -718,8 +701,18 @@ class TestJunebugApi(JunebugTestBase):
 
     @inlineCallbacks
     def test_send_message_no_to_or_reply_to(self):
+        channel = Channel(
+            redis_manager=(yield self.get_redis()),
+            config=(yield self.create_channel_config()),
+            properties=self.create_channel_properties(),
+            id='test-channel')
+
+        yield channel.save()
+        yield channel.start(self.service)
+
         resp = yield self.post(
-            '/channels/foo-bar/messages/', {'from': None, 'content': None})
+            '/channels/{}/messages/'.format(channel.id),
+            {'from': None, 'content': None})
         yield self.assert_response(
             resp, http.BAD_REQUEST, 'api usage error', {
                 'errors': [{
@@ -728,6 +721,33 @@ class TestJunebugApi(JunebugTestBase):
                 }]
             })
 
+    @inlineCallbacks
+    def test_send_message_no_destination(self):
+        '''Sending a message on a channel endpoint without a destination should
+        raise a ApiUsageError
+        '''
+        properties = self.create_channel_properties()
+        del properties['mo_url']
+        channel = Channel(
+            redis_manager=(yield self.get_redis()),
+            config=(yield self.create_channel_config()),
+            properties=properties,
+            id='test-channel')
+
+        yield channel.save()
+        yield channel.start(self.service)
+
+        resp = yield self.post(
+            '/channels/{}/messages/'.format(channel.id),
+            {'from': None, 'content': 'test message', 'to': '+1234'})
+        yield self.assert_response(
+            resp, http.BAD_REQUEST, 'api usage error', {
+                'errors': [{
+                    'message': 'This channel has no "mo_url" or "amqp_queue"',
+                    'type': 'ApiUsageError',
+                }]
+            })
+
     @inlineCallbacks
     def test_send_message_additional_properties(self):
         '''Additional properties should result in an error being returned.'''
@@ -897,7 +917,16 @@ class TestJunebugApi(JunebugTestBase):
     @inlineCallbacks
     def test_get_message_status_no_events(self):
         '''Returns `None` for last event fields, and empty list for events'''
-        resp = yield self.get('/channels/foo-bar/messages/message-id')
+
+        properties = self.create_channel_properties(character_limit=10)
+        config = yield self.create_channel_config()
+        redis = yield self.get_redis()
+        channel = Channel(redis, config, properties, id='test-channel')
+        yield channel.save()
+        yield channel.start(self.service)
+
+        resp = yield self.get(
+            '/channels/{}/messages/message-id'.format(channel.id))
         yield self.assert_response(
             resp, http.OK, 'message status', {
                 'id': 'message-id',
@@ -906,16 +935,47 @@ class TestJunebugApi(JunebugTestBase):
                 'events': [],
             })
 
+    @inlineCallbacks
+    def test_get_message_status_with_no_destination(self):
+        '''Returns error if the channel has no destination'''
+        properties = self.create_channel_properties(character_limit=10)
+        del properties['mo_url']
+        config = yield self.create_channel_config()
+        redis = yield self.get_redis()
+        channel = Channel(redis, config, properties, id='test-channel')
+        yield channel.save()
+        yield channel.start(self.service)
+
+        resp = yield self.get(
+            '/channels/{}/messages/message-id'.format(channel.id))
+
+        yield self.assert_response(
+            resp, http.BAD_REQUEST, 'api usage error', {
+                'errors': [{
+                    'message': 'This channel has no "mo_url" or "amqp_queue"',
+                    'type': 'ApiUsageError',
+                }]
+            })
+
     @inlineCallbacks
     def test_get_message_status_one_event(self):
         '''Returns the event details for last event fields, and list with
         single event for `events`'''
+
+        properties = self.create_channel_properties(character_limit=10)
+        config = yield self.create_channel_config()
+        redis = yield self.get_redis()
+        channel = Channel(redis, config, properties, id='test-channel')
+        yield channel.save()
+        yield channel.start(self.service)
+
         event = TransportEvent(
             user_message_id='message-id', sent_message_id='message-id',
             event_type='nack', nack_reason='error error')
-        yield self.outbounds.store_event('channel-id', 'message-id', event)
-        resp = yield self.get('/channels/channel-id/messages/message-id')
-        event_dict = api_from_event('channel-id', event)
+        yield self.outbounds.store_event(channel.id, 'message-id', event)
+        resp = yield self.get(
+            '/channels/{}/messages/message-id'.format(channel.id))
+        event_dict = api_from_event(channel.id, event)
         event_dict['timestamp'] = str(event_dict['timestamp'])
         yield self.assert_response(
             resp, http.OK, 'message status', {
@@ -929,19 +989,28 @@ class TestJunebugApi(JunebugTestBase):
     def test_get_message_status_multiple_events(self):
         '''Returns the last event details for last event fields, and list with
         all events for `events`'''
+
+        properties = self.create_channel_properties(character_limit=10)
+        config = yield self.create_channel_config()
+        redis = yield self.get_redis()
+        channel = Channel(redis, config, properties, id='test-channel')
+        yield channel.save()
+        yield channel.start(self.service)
+
         events = []
         event_dicts = []
         for i in range(5):
             event = TransportEvent(
                 user_message_id='message-id', sent_message_id='message-id',
                 event_type='nack', nack_reason='error error')
-            yield self.outbounds.store_event('channel-id', 'message-id', event)
+            yield self.outbounds.store_event(channel.id, 'message-id', event)
             events.append(event)
-            event_dict = api_from_event('channel-id', event)
+            event_dict = api_from_event(channel.id, event)
             event_dict['timestamp'] = str(event_dict['timestamp'])
             event_dicts.append(event_dict)
 
-        resp = yield self.get('/channels/channel-id/messages/message-id')
+        resp = yield self.get(
+            '/channels/{}/messages/message-id'.format(channel.id))
         yield self.assert_response(
             resp, http.OK, 'message status', {
                 'id': 'message-id',
@@ -975,7 +1044,7 @@ class TestJunebugApi(JunebugTestBase):
             request_list.append(
                 ((b'get', url, mock.ANY, mock.ANY, mock.ANY),
                  (http.OK, {b'Content-Type': b'application/json'},
-                  b'{"messages": 1256, "messages_details": {"rate": 1.25}, "name": "%s"}' % queue_name)))  # noqa
+                  b'{"messages": 1256, "messages_stats": {"ack_details": {"rate": 1.25}}, "name": "%s"}' % queue_name)))  # noqa
 
         async_failures = []
         sequence_stubs = RequestSequence(request_list, async_failures.append)
@@ -1008,6 +1077,60 @@ class TestJunebugApi(JunebugTestBase):
                         'stuck': False
                     }])
 
+    @inlineCallbacks
+    def test_get_channels_and_destinations_health_check(self):
+
+        config = yield self.create_channel_config(
+            rabbitmq_management_interface="rabbitmq:15672"
+        )
+        yield self.stop_server()
+        yield self.start_server(config=config)
+
+        channel = yield self.create_channel(self.service, self.redis)
+
+        config = self.create_router_config()
+        router = Router(self.api, config)
+        dest_config = self.create_destination_config()
+        destination = router.add_destination(dest_config)
+        dest_config = self.create_destination_config()
+        destination2 = router.add_destination(dest_config)
+        router.save()
+
+        destinations = sorted([destination.id, destination2.id])
+
+        request_list = []
+        response = []
+
+        for queue_id in [channel.id] + destinations:
+            for sub in ['inbound', 'outbound', 'event']:
+                queue_name = "%s.%s" % (queue_id, sub)
+                url = 'http://rabbitmq:15672/api/queues/%%2F/%s' % (queue_name)
+                request_list.append(
+                    ((b'get', url, mock.ANY, mock.ANY, mock.ANY),
+                     (http.OK, {b'Content-Type': b'application/json'},
+                      b'{"messages": 1256, "messages_stats": {"ack_details": {"rate": 1.25}}, "name": "%s"}' % queue_name)))  # noqa
+
+                response.append({
+                    'messages': 1256,
+                    'name': '{}.{}'.format(queue_id, sub),
+                    'rate': 1.25,
+                    'stuck': False
+                })
+
+        async_failures = []
+        sequence_stubs = RequestSequence(request_list, async_failures.append)
+        stub_treq = StubTreq(StringStubbingResource(sequence_stubs))
+
+        def new_get(*args, **kwargs):
+            return stub_treq.request("GET", args[0])
+
+        with (mock.patch('treq.client.HTTPClient.get', side_effect=new_get)):
+            with sequence_stubs.consume(self.fail):
+                resp = yield self.request('GET', '/health')
+
+            yield self.assertEqual(async_failures, [])
+            yield self.assert_response(resp, http.OK, 'queues ok', response)
+
     @inlineCallbacks
     def test_get_channels_health_check_stuck(self):
 
@@ -1027,7 +1150,7 @@ class TestJunebugApi(JunebugTestBase):
             request_list.append(
                 ((b'get', url, mock.ANY, mock.ANY, mock.ANY),
                  (http.OK, {b'Content-Type': b'application/json'},
-                  b'{"messages": 1256, "messages_details": {"rate": 0}, "name": "%s"}' % queue_name)))  # noqa
+                  b'{"messages": 1256, "messages_stats": {"ack_details": {"rate": 0.0}}, "name": "%s"}' % queue_name)))  # noqa
 
         async_failures = []
         sequence_stubs = RequestSequence(request_list, async_failures.append)
@@ -1264,8 +1387,8 @@ class TestJunebugApi(JunebugTestBase):
         self.assertEqual(transport.parent, self.service)
 
         worker_config = config['config']
-        worker_config['destinations'] = []
-        self.assertEqual(transport.config, worker_config)
+        for k, v in worker_config.items():
+            self.assertEqual(transport.config[k], worker_config[k])
 
     @inlineCallbacks
     def test_create_router_saves_config(self):
@@ -1316,8 +1439,8 @@ class TestJunebugApi(JunebugTestBase):
         self.assertEqual(router_config, old_config)
         router_worker = self.api.service.namedServices[router_id]
         router_worker_config = old_config['config']
-        router_worker_config['destinations'] = []
-        self.assertEqual(router_worker.config, router_worker_config)
+        for k, v in router_worker_config.items():
+            self.assertEqual(router_worker.config[k], router_worker_config[k])
 
         new_config = self.create_router_config(config={'test': 'pass'})
         new_config.pop('label', None)
@@ -1332,8 +1455,8 @@ class TestJunebugApi(JunebugTestBase):
         self.assertEqual(router_config, new_config)
         router_worker = self.api.service.namedServices[router_id]
         router_worker_config = new_config['config']
-        router_worker_config['destinations'] = []
-        self.assertEqual(router_worker.config, router_worker_config)
+        for k, v in router_worker_config.items():
+            self.assertEqual(router_worker.config[k], router_worker_config[k])
 
         router_worker = self.api.service.namedServices[router_id]
 
@@ -1375,8 +1498,8 @@ class TestJunebugApi(JunebugTestBase):
         self.assertEqual(router_config, old_config)
         router_worker = self.api.service.namedServices[router_id]
         router_worker_config = old_config['config']
-        router_worker_config['destinations'] = []
-        self.assertEqual(router_worker.config, router_worker_config)
+        for k, v in router_worker_config.items():
+            self.assertEqual(router_worker.config[k], router_worker_config[k])
 
         update = {'config': {'test': 'pass', 'new': 'new'}}
         new_config = deepcopy(old_config)
@@ -1393,8 +1516,8 @@ class TestJunebugApi(JunebugTestBase):
         self.assertEqual(router_config, new_config)
         router_worker = self.api.service.namedServices[router_id]
         router_worker_config = new_config['config']
-        router_worker_config['destinations'] = []
-        self.assertEqual(router_worker.config, new_config['config'])
+        for k, v in router_worker_config.items():
+            self.assertEqual(router_worker.config[k], router_worker_config[k])
 
         router_worker = self.api.service.namedServices[router_id]
 
@@ -1422,6 +1545,34 @@ class TestJunebugApi(JunebugTestBase):
         yield self.assert_response(
             resp, http.OK, 'router found', old_config, ignore=['id'])
 
+    @inlineCallbacks
+    def test_update_from_address_router_config(self):
+        """When creating a PATCH request, the from address router configuration
+        should be updated"""
+
+        resp = yield self.post('/channels/', {
+            'type': 'telnet',
+            'config': {
+                'twisted_endpoint': 'tcp:0',
+            }
+        })
+        channel_id = (yield resp.json())['result']['id']
+
+        old_config = self.create_router_config(
+            label='old', type='from_address',
+            config={'channel': channel_id})
+        resp = yield self.post('/routers/', old_config)
+        router_id = (yield resp.json())['result']['id']
+
+        update = {'config': {'channel': channel_id}}
+        new_config = deepcopy(old_config)
+        new_config.update(update)
+        resp = yield self.patch_request(
+            '/routers/{}'.format(router_id), new_config)
+
+        yield self.assert_response(
+            resp, http.OK, 'router updated', new_config, ignore=['id'])
+
     @inlineCallbacks
     def test_delete_router(self):
         """Should stop the router from running, and delete its config"""
@@ -1450,6 +1601,135 @@ class TestJunebugApi(JunebugTestBase):
             }]
         })
 
+    @inlineCallbacks
+    def test_get_router_logs_no_logs(self):
+        '''If there are no logs, an empty list should be returned.'''
+        router = yield self.create_test_router(self.service)
+        log_worker = router.router_worker.getServiceNamed(
+            'Junebug Worker Logger')
+        yield log_worker.startService()
+        resp = yield self.get('/routers/%s/logs' % router.id, params={
+            'n': '3',
+        })
+        self.assert_response(
+            resp, http.OK, 'logs retrieved', [])
+
+    @inlineCallbacks
+    def test_get_router_logs_less_than_limit(self):
+        '''If the amount of logs is less than the limit, all the logs should
+        be returned.'''
+        router = yield self.create_test_router(self.service)
+        worker_logger = router.router_worker.getServiceNamed(
+            'Junebug Worker Logger')
+        worker_logger.startService()
+
+        router.router_worker.test_log('Test')
+        resp = yield self.get('/routers/%s/logs' % router.id, params={
+            'n': '2',
+        })
+        self.assert_response(
+            resp, http.OK, 'logs retrieved', [], ignore=[0])
+        [log] = (yield resp.json())['result']
+        self.assert_log(log, {
+            'logger': router.id,
+            'message': 'Test',
+            'level': logging.INFO})
+
+    @inlineCallbacks
+    def test_get_router_logs_more_than_limit(self):
+        '''If the amount of logs is more than the limit, only the latest n
+        should be returned.'''
+        router = yield self.create_test_router(self.service)
+        worker_logger = router.router_worker.getServiceNamed(
+            'Junebug Worker Logger')
+        worker_logger.startService()
+
+        router.router_worker.test_log('Test1')
+        router.router_worker.test_log('Test2')
+        router.router_worker.test_log('Test3')
+        resp = yield self.get('/routers/%s/logs' % router.id, params={
+            'n': '2',
+        })
+        self.assert_response(
+            resp, http.OK, 'logs retrieved', [], ignore=[1, 0])
+        [log1, log2] = (yield resp.json())['result']
+        self.assert_log(log1, {
+            'logger': router.id,
+            'message': 'Test3',
+            'level': logging.INFO})
+        self.assert_log(log2, {
+            'logger': router.id,
+            'message': 'Test2',
+            'level': logging.INFO})
+
+    @inlineCallbacks
+    def test_get_router_logs_more_than_configured(self):
+        '''If the amount of requested logs is more than what is
+        configured, then only the configured amount of logs are returned.'''
+        logpath = self.mktemp()
+        config = yield self.create_channel_config(
+            max_logs=2,
+            logging_path=logpath
+        )
+        yield self.stop_server()
+        yield self.start_server(config=config)
+        router = yield self.create_test_router(self.service)
+        worker_logger = router.router_worker.getServiceNamed(
+            'Junebug Worker Logger')
+        worker_logger.startService()
+
+        router.router_worker.test_log('Test1')
+        router.router_worker.test_log('Test2')
+        router.router_worker.test_log('Test3')
+        resp = yield self.get('/routers/%s/logs' % router.id, params={
+            'n': '3',
+        })
+
+        self.assert_response(
+            resp, http.OK, 'logs retrieved', [], ignore=[1, 0])
+        [log1, log2] = (yield resp.json())['result']
+        self.assert_log(log1, {
+            'logger': router.id,
+            'message': 'Test3',
+            'level': logging.INFO})
+        self.assert_log(log2, {
+            'logger': router.id,
+            'message': 'Test2',
+            'level': logging.INFO})
+
+    @inlineCallbacks
+    def test_get_router_logs_no_n(self):
+        '''If the number of logs is not specified, then the API should return
+        the configured maximum number of logs.'''
+        logpath = self.mktemp()
+        config = yield self.create_channel_config(
+            max_logs=2,
+            logging_path=logpath
+        )
+        yield self.stop_server()
+        yield self.start_server(config=config)
+        router = yield self.create_test_router(self.service)
+        worker_logger = router.router_worker.getServiceNamed(
+            'Junebug Worker Logger')
+        worker_logger.startService()
+
+        router.router_worker.test_log('Test1')
+        router.router_worker.test_log('Test2')
+        router.router_worker.test_log('Test3')
+        resp = yield self.get('/routers/%s/logs' % router.id)
+
+        self.assert_response(
+            resp, http.OK, 'logs retrieved', [], ignore=[1, 0])
+        [log1, log2] = (yield resp.json())['result']
+        self.assert_log(log1, {
+            'logger': router.id,
+            'message': 'Test3',
+            'level': logging.INFO})
+        self.assert_log(log2, {
+            'logger': router.id,
+            'message': 'Test2',
+            'level': logging.INFO})
+
     @inlineCallbacks
     def test_create_destination_invalid_router_id(self):
         """If the router specified by the router ID doesn't exist, a not found
@@ -1494,7 +1774,14 @@ class TestJunebugApi(JunebugTestBase):
         resp = yield self.post('/routers/', router_config)
         router_id = (yield resp.json())['result']['id']
 
-        dest_config = self.create_destination_config()
+        dest_config = self.create_destination_config(
+            label='testlabel',
+            metadata={'test': 'metadata'},
+            mo_url='http//example.org',
+            mo_url_token='12345',
+            amqp_queue='testqueue',
+            character_limit=7,
+        )
         resp = yield self.post(
             '/routers/{}/destinations/'.format(router_id), dest_config)
         self.assert_response(
@@ -1611,7 +1898,7 @@ class TestJunebugApi(JunebugTestBase):
         destination = router_worker.config['destinations'][0]
         self.assertIn('label', destination)
 
-        new_config = self.create_destination_config()
+        new_config = self.create_destination_config(character_limit=7)
         resp = yield self.put(
             '/routers/{}/destinations/{}'.format(router_id, destination_id),
             new_config)
@@ -1692,10 +1979,10 @@ class TestJunebugApi(JunebugTestBase):
 
         resp = yield self.patch_request(
             '/routers/{}/destinations/{}'.format(router_id, destination_id),
-            {'metadata': {'foo': 'bar'}})
+            {'metadata': {'foo': 'bar'}, 'character_limit': 7})
         self.assert_response(
             resp, http.OK, 'destination updated', dest_config,
-            ignore=['id', 'metadata'])
+            ignore=['id', 'metadata', 'character_limit'])
 
         router_worker = self.api.service.namedServices[router_id]
         destination = router_worker.config['destinations'][0]
@@ -1795,3 +2082,285 @@ class TestJunebugApi(JunebugTestBase):
                     'type': "DestinationNotFound",
                 }]
             })
+
+    @inlineCallbacks
+    def test_send_destination_message_invalid_router(self):
+        resp = yield self.post(
+            '/routers/foo-bar/destinations/test-destination/messages/',
+            {'to': '+1234', 'from': '', 'content': None})
+        yield self.assert_response(
+            resp, http.NOT_FOUND, 'router not found', {
+                'errors': [{
+                    'message': 'Router with ID foo-bar cannot be found',
+                    'type': 'RouterNotFound',
+                    }]
+                })
+
+    @inlineCallbacks
+    def test_send_destination_message_invalid_destination(self):
+        router_config = self.create_router_config()
+        resp = yield self.post('/routers/', router_config)
+        router_id = (yield resp.json())['result']['id']
+
+        resp = yield self.post(
+            '/routers/{}/destinations/test-destination/messages/'.format(
+                router_id),
+            {'to': '+1234', 'from': '', 'content': None})
+        yield self.assert_response(
+            resp, http.NOT_FOUND, 'destination not found', {
+                'errors': [{
+                    'message':
+                        'Cannot find destination with ID test-destination for '
+                        'router {}'.format(router_id),
+                    'type': 'DestinationNotFound',
+                    }]
+                })
+
+    @inlineCallbacks
+    def test_send_destination_message(self):
+        '''Sending a message should place the message on the queue for the
+        channel'''
+        properties = self.create_channel_properties()
+        config = yield self.create_channel_config()
+        redis = yield self.get_redis()
+        channel = Channel(redis, config, properties, id='test-channel')
+        yield channel.save()
+        yield channel.start(self.service)
+
+        router_config = self.create_router_config()
+        resp = yield self.post('/routers/', router_config)
+        router_id = (yield resp.json())['result']['id']
+
+        dest_config = self.create_destination_config(
+            config={'channel': 'test-channel'})
+        resp = yield self.post(
+            '/routers/{}/destinations/'.format(router_id), dest_config)
+        destination_id = (yield resp.json())['result']['id']
+
+        resp = yield self.post(
+            '/routers/{}/destinations/{}/messages/'.format(
+                router_id, destination_id),
+            {'to': '+1234', 'content': 'foo', 'from': None})
+        yield self.assert_response(
+            resp, http.CREATED, 'message submitted', {
+                'to': '+1234',
+                'channel_id': 'test-channel',
+                'from': None,
+                'group': None,
+                'reply_to': None,
+                'channel_data': {},
+                'content': 'foo',
+            }, ignore=['timestamp', 'message_id'])
+
+        [message] = self.get_dispatched_messages('test-channel.outbound')
+        message_id = (yield resp.json())['result']['message_id']
+        self.assertEqual(message['message_id'], message_id)
+
+        event_url = yield self.api.outbounds.load_event_url(
+            'test-channel', message['message_id'])
+        self.assertEqual(event_url, None)
+
+    @inlineCallbacks
+    def test_send_destination_message_reply(self):
+        '''Sending a reply message should fetch the relevant inbound message,
+        use it to construct a reply message, and place the reply message on the
+        queue for the channel'''
+        properties = self.create_channel_properties()
+        config = yield self.create_channel_config()
+        redis = yield self.get_redis()
+        channel = Channel(redis, config, properties, id='test-channel')
+        yield channel.save()
+        yield channel.start(self.service)
+
+        router_config = self.create_router_config()
+        resp = yield self.post('/routers/', router_config)
+        router_id = (yield resp.json())['result']['id']
+
+        dest_config = self.create_destination_config(
+            config={'channel': 'test-channel'})
+        resp = yield self.post(
+            '/routers/{}/destinations/'.format(router_id), dest_config)
+        destination_id = (yield resp.json())['result']['id']
+
+        in_msg = TransportUserMessage(
+            from_addr='+2789',
+            to_addr='+1234',
+            transport_name='test-channel',
+            transport_type='_',
+            transport_metadata={'foo': 'bar'})
+
+        yield self.api.inbounds.store_vumi_message(destination_id, in_msg)
+        expected = in_msg.reply(content='testcontent')
+        expected = api_from_message(expected)
+
+        resp = yield self.post(
+            '/routers/{}/destinations/{}/messages/'.format(
+                router_id, destination_id),
+            {'reply_to': in_msg['message_id'], 'content': 'testcontent'})
+
+        yield self.assert_response(
+            resp, http.CREATED,
+            'message submitted',
+            omit(expected, 'timestamp', 'message_id'),
+            ignore=['timestamp', 'message_id'])
+
+        [message] = self.get_dispatched_messages('test-channel.outbound')
+        message_id = (yield resp.json())['result']['message_id']
+        self.assertEqual(message['message_id'], message_id)
+
+        event_url = yield self.api.outbounds.load_event_url(
+            'test-channel', message['message_id'])
+        self.assertEqual(event_url, None)
+
+        # router_worker = self.api.service.namedServices[router_id]
+        stored_message = yield self.api.outbounds.load_message(
+            destination_id, message['message_id'])
+
+        self.assertEqual(api_from_message(message), stored_message)
+
+    @inlineCallbacks
+    def test_get_destination_message_invalid_router(self):
+        resp = yield self.get(
+            '/routers/foo-bar/destinations/test-destination/messages/message-id')  # noqa
+        yield self.assert_response(
+            resp, http.NOT_FOUND, 'router not found', {
+                'errors': [{
+                    'message': 'Router with ID foo-bar cannot be found',
+                    'type': 'RouterNotFound',
+                    }]
+                })
+
+    @inlineCallbacks
+    def test_get_destination_message_invalid_destination(self):
+        router_config = self.create_router_config()
+        resp = yield self.post('/routers/', router_config)
+        router_id = (yield resp.json())['result']['id']
+
+        resp = yield self.get(
+            '/routers/{}/destinations/test-destination/messages/message-id'.format(  # noqa
+                router_id))
+        yield self.assert_response(
+            resp, http.NOT_FOUND, 'destination not found', {
+                'errors': [{
+                    'message':
+                        'Cannot find destination with ID test-destination for '
+                        'router {}'.format(router_id),
+                    'type': 'DestinationNotFound',
+                    }]
+                })
+
+    @inlineCallbacks
+    def test_get_destination_message_status_no_events(self):
+        '''Returns `None` for last event fields, and empty list for events'''
+        router_config = self.create_router_config()
+        resp = yield self.post('/routers/', router_config)
+        router_id = (yield resp.json())['result']['id']
+
+        dest_config = self.create_destination_config(config={'channel': '123'})
+        resp = yield self.post(
+            '/routers/{}/destinations/'.format(router_id), dest_config)
+        destination_id = (yield resp.json())['result']['id']
+
+        resp = yield self.get(
+            '/routers/{}/destinations/{}/messages/message-id'.format(
+                router_id, destination_id))
+        yield self.assert_response(
+            resp, http.OK, 'message status', {
+                'id': 'message-id',
+                'last_event_type': None,
+                'last_event_timestamp': None,
+                'events': [],
+            })
+
+    @inlineCallbacks
+    def test_get_destination_message_status_with_events(self):
+        '''Returns `None` for last event fields, and empty list for events'''
+        router_config = self.create_router_config()
+        resp = yield self.post('/routers/', router_config)
+        router_id = (yield resp.json())['result']['id']
+
+        dest_config = self.create_destination_config(
+            config={'channel': 'channel-id'})
+        resp = yield self.post(
+            '/routers/{}/destinations/'.format(router_id), dest_config)
+        destination_id = (yield resp.json())['result']['id']
+
+        resp = yield self.get(
+            '/routers/{}/destinations/{}/messages/message-id'.format(
+                router_id, destination_id))
+        yield self.assert_response(
+            resp, http.OK, 'message status', {
+                'id': 'message-id',
+                'last_event_type': None,
+                'last_event_timestamp': None,
+                'events': [],
+            })
+
+    @inlineCallbacks
+    def test_get_destination_message_status_one_event(self):
+        '''Returns the event details for last event fields, and list with
+        single event for `events`'''
+        router_config = self.create_router_config()
+        resp = yield self.post('/routers/', router_config)
+        router_id = (yield resp.json())['result']['id']
+
+        dest_config = self.create_destination_config(
+            config={'channel': 'channel-id'})
+        resp = yield self.post(
+            '/routers/{}/destinations/'.format(router_id), dest_config)
+        destination_id = (yield resp.json())['result']['id']
+
+        event = TransportEvent(
+            user_message_id='message-id', sent_message_id='message-id',
+            event_type='nack', nack_reason='error error')
+        yield self.outbounds.store_event(destination_id, 'message-id', event)
+        resp = yield self.get(
+            '/routers/{}/destinations/{}/messages/message-id'.format(
+                router_id, destination_id))
+        event_dict = api_from_event(destination_id, event)
+        event_dict['timestamp'] = str(event_dict['timestamp'])
+        yield self.assert_response(
+            resp, http.OK, 'message status', {
+                'id': 'message-id',
+                'last_event_type': 'rejected',
+                'last_event_timestamp': str(event['timestamp']),
+                'events': [event_dict],
+            })
+
+    @inlineCallbacks
+    def test_get_destination_message_status_multiple_events(self):
+        '''Returns the last event details for last event fields, and list with
+        all events for `events`'''
+        router_config = self.create_router_config()
+        resp = yield self.post('/routers/', router_config)
+        router_id = (yield resp.json())['result']['id']
+
+        dest_config = self.create_destination_config(
+            config={'channel': 'channel-id'})
+        resp = yield self.post(
+            '/routers/{}/destinations/'.format(router_id), dest_config)
+        destination_id = (yield resp.json())['result']['id']
+
+        events = []
+        event_dicts = []
+        for i in range(5):
+            event = TransportEvent(
+                user_message_id='message-id', sent_message_id='message-id',
+                event_type='nack', nack_reason='error error')
+            yield self.outbounds.store_event(
+                destination_id, 'message-id', event)
+            events.append(event)
+            event_dict = api_from_event(destination_id, event)
+            event_dict['timestamp'] = str(event_dict['timestamp'])
+            event_dicts.append(event_dict)
+
+        resp = yield self.get(
+            '/routers/{}/destinations/{}/messages/message-id'.format(
+                router_id, destination_id))
+        yield self.assert_response(
+            resp, http.OK, 'message status', {
+                'id': 'message-id',
+                'last_event_type': 'rejected',
+                'last_event_timestamp': event_dicts[-1]['timestamp'],
+                'events': event_dicts,
+            })
